==> dna-0.5.2.tar.gz: lib/src/meta.dart <==
//  Copyright 2016 root.ext@gmail.com
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import 'dart:mirrors';
import 'native.dart';
import 'util.dart';
import 'model/library.dart';
import 'model/library_meta.dart';
import 'model/method.dart';
import 'model/method_meta.dart';
import 'model/param.dart';
import 'model/pointer.dart';

final int NULL_POINTER = 0;

LibraryMeta getLibrary(Type type, Resolver resolver) {
  LibraryMeta library = new LibraryMeta();
  library.type = type;
  library.resolver = resolver;

  var libTag = findClassTag(reflectClass(type), Library);
  assert(libTag != null, 'class has no @Library');
  var name = (libTag.getField(#name).reflectee as String);
  var altName = (libTag.getField(#altName).reflectee as String);
  assert(name != null, 'library has no name');

  library.pointer = loadLibrary(name);
  if (library.pointer == NULL_POINTER && altName != null) {
    library.pointer = loadLibrary(altName);
  }
  //print('library $name : ${library.pointer}');
  library.methods = new Map<String, MethodMeta>();

  return library;
}

MethodMeta getMethod(Invocation invocation, LibraryMeta library) {
  String name = MirrorSystem.getName(invocation.memberName);
  if (library.methods.containsKey(name)) {
    return library.methods[name];
  }

  var classMirror = reflectClass(library.type);
  var methods = findMethods(classMirror, invocation.memberName);
  if (methods.isEmpty && classMirror.superinterfaces.isNotEmpty){
    methods = findMethods(classMirror.superinterfaces.first, invocation.memberName);
  }

  assert(methods.isNotEmpty, 'cannot find method ${invocation.memberName}');
  var methodMirror = methods.first;

  MethodMeta methodMeta = new MethodMeta();
  methodMeta.name = MirrorSystem.getName(methodMirror.simpleName);

  methodMeta.params = new List<Param>();
  var paramMirrors = methodMirror.parameters;

  var types = new List<int>();
  for (var paramMirror in paramMirrors) {
    var paramTag = findParamTag(paramMirror, Param);
    var param = paramTag != null
        ? (paramTag.reflectee as Param)
        : new Param(dartTypeToCtype(paramMirror.type),
            ref: paramMirror.type.isSubtypeOf(reflectType(Pointer)));
    types.add(paramType(param));

    methodMeta.params.add(param);
  }

  methodMeta.types = types;

  var methodTag = findMethodTag(methodMirror, Method);
  var retType = methodTag != null
      ? (methodTag.getField(#type).reflectee as int)
      : dartTypeToCtype(methodMirror.returnType);
  var methodName = methodTag != null
      ? (methodTag.getField(#name).reflectee as String)
      : null;

  methodMeta.name = methodName != null ? methodName : name;
  methodMeta.returnParam = new Param(retType);
  methodMeta.pointer = resolveMethod(invocation, library, methodMeta.name);

  library.methods[name] = methodMeta;
  return methodMeta;
}

List<DeclarationMirror> findMethods(ClassMirror classMirror, Symbol name) {
  var methods = classMirror.declarations.values
      .where((e) =>
          /*e.isRegularMethod && e.owner == classMirror &&*/
          e.simpleName == name)
      .toList();
  return methods;
}

int resolveMethod(Invocation invocation, LibraryMeta library, String name) {
  if (library.resolver != null) {
    var pointer = library.resolver(invocation);
    if (pointer == NULL_POINTER) {
      pointer = findSymbol(library.pointer, name);
    }
    return pointer;
  }

  var methodPointer = findSymbol(library.pointer, name);
  return methodPointer;
}

int paramType(Param param) {
  int type = param.type;

  if (param.list) {
    type = TYPEDDATA;
  }

  switch (param.type) {
    case C_STRING:
      {
        type = TYPEDDATA;
      }
      break;
    case C_STRUCT:
      {
        type = TYPEDDATA;
      }
      break;
  }

  return type;
}

int dartTypeToCtype(TypeMirror type) {
  if (type == reflectClass(int)) {
    return C_INT;
  } else if (type == reflectClass(double)) {
    return C_DOUBLE;
  } else if (type == reflectClass(bool)) {
    return C_BOOL;
  } else if (type == reflectClass(String)) {
    return C_STRING;
  } else if (type.isSubtypeOf(reflectType(Pointer))) {
    return C_POINTER;
  } else {
    return VOID;
  }
}

==> dna-0.5.3.tar.gz: lib/src/meta.dart <==
//  Copyright 2016 root.ext@gmail.com
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import 'dart:mirrors';
import 'native.dart';
import 'util.dart';
import 'model/library.dart';
import 'model/library_meta.dart';
import 'model/method.dart';
import 'model/method_meta.dart';
import 'model/param.dart';
import 'model/pointer.dart';

final int NULL_POINTER = 0;

LibraryMeta getLibrary(Type type, Resolver resolver) {
  LibraryMeta library = new LibraryMeta();
  library.type = type;
  library.resolver = resolver;

  var libTag = findClassTag(reflectClass(type), Library);
  assert(libTag != null, 'class has no @Library');
  var name = (libTag.getField(#name).reflectee as String);
  var altName = (libTag.getField(#altName).reflectee as String);
  assert(name != null, 'library has no name');

  library.pointer = loadLibrary(name);
  if (library.pointer == NULL_POINTER && altName != null) {
    library.pointer = loadLibrary(altName);
  }
  assert(library.pointer != NULL_POINTER, 'cannot load library $name');
  library.methods = new Map<String, MethodMeta>();

  return library;
}

MethodMeta getMethod(Invocation invocation, LibraryMeta library) {
  String name = MirrorSystem.getName(invocation.memberName);
  if (library.methods.containsKey(name)) {
    return library.methods[name];
  }

  var classMirror = reflectClass(library.type);
  var methods = findMethods(classMirror, invocation.memberName);
  if (methods.isEmpty && classMirror.superinterfaces.isNotEmpty) {
    methods = findMethods(classMirror.superinterfaces.first, invocation.memberName);
  }

  assert(methods.isNotEmpty, 'cannot find method ${invocation.memberName}');
  var methodMirror = methods.first;

  MethodMeta methodMeta = new MethodMeta();
  methodMeta.name = MirrorSystem.getName(methodMirror.simpleName);

  methodMeta.params = new List<Param>();
  var paramMirrors = methodMirror.parameters;

  var types = new List<int>();
  for (var paramMirror in paramMirrors) {
    var paramTag = findParamTag(paramMirror, Param);
    var param = paramTag != null
        ? (paramTag.reflectee as Param)
        : new Param(dartTypeToCtype(paramMirror.type),
            ref: paramMirror.type.isSubtypeOf(reflectType(Pointer)));
    types.add(paramType(param));

    methodMeta.params.add(param);
  }

  methodMeta.types = types;

  var methodTag = findMethodTag(methodMirror, Method);
  var retType = methodTag != null
      ? (methodTag.getField(#type).reflectee as int)
      : dartTypeToCtype(methodMirror.returnType);
  var methodName = methodTag != null
      ? (methodTag.getField(#name).reflectee as String)
      : null;

  methodMeta.name = methodName != null ? methodName : name;
  methodMeta.returnParam = new Param(retType);
  methodMeta.pointer = resolveMethod(invocation, library, methodMeta.name);

  library.methods[name] = methodMeta;
  return methodMeta;
}

List<MethodMirror> findMethods(ClassMirror classMirror, Symbol name) {
  var methods = classMirror.declarations.values
      .where((e) => e.simpleName == name)
      .where((e) => e is MethodMirror)
      .map((e) => e as MethodMirror)
      .toList();
  return methods;
}

int resolveMethod(Invocation invocation, LibraryMeta library, String name) {
  if (library.resolver != null) {
    var pointer = library.resolver(invocation);
    if (pointer == NULL_POINTER) {
      pointer = findSymbol(library.pointer, name);
    }
    return pointer;
  }

  var methodPointer = findSymbol(library.pointer, name);
  return methodPointer;
}

int paramType(Param param) {
  int type = param.type;

  if (param.list) {
    type = TYPEDDATA;
  }

  switch (param.type) {
    case C_STRING:
      {
        type = TYPEDDATA;
      }
      break;
    case C_STRUCT:
      {
        type = TYPEDDATA;
      }
      break;
  }

  return type;
}

int dartTypeToCtype(TypeMirror type) {
  if (type == reflectClass(int)) {
    return C_INT;
  } else if (type == reflectClass(double)) {
    return C_DOUBLE;
  } else if (type == reflectClass(bool)) {
    return C_BOOL;
  } else if (type == reflectClass(String)) {
    return C_STRING;
  } else if (type.isSubtypeOf(reflectType(Pointer))) {
    return C_POINTER;
  } else {
    return VOID;
  }
}
